// Win32 Tutorial (Console Debug Window)

// Alan Baylis 2004

#include <windows.h>
#include <stdio.h>
#include <iostream>
#include "console.h"
#include "resource.h"

const char ClassName[] = "MainWindowClass";

bool key[256];
bool released_key[256];

ConsoleWindow Console;

LRESULT CALLBACK WndProc( HWND    hWnd,
                          UINT    Msg,
                          WPARAM  wParam,          
                          LPARAM  lParam )
{
    switch (Msg)
    {
        case WM_COMMAND: 
        { 
            switch(LOWORD(wParam))
            { 
                case IDM_FILE_EXIT: 
                    PostMessage(hWnd, WM_CLOSE, 0, 0);
                break; 

                case IDM_OPEN: 
                    Console.Open();
                break; 

                case IDM_CLOSE: 
                    Console.Close();
                break; 

                case IDM_TEST: 
                {
                    int iVar;

                    // test stdio
                    fprintf(stdout, "Test output to stdout\n");
                    fprintf(stderr, "Test output to stderr\n");
                    fprintf(stdout, "Enter an integer to test stdin: ");
                    scanf("%d", &iVar);
                    printf("You entered %d\n", iVar);

                    //test iostreams
                    cout << "Test output to cout" << endl;
                    cerr << "Test output to cerr" << endl;
                    clog << "Test output to clog" << endl;
                    cout << "Enter an integer to test cin: ";
                    cin >> iVar;
                    cout << "You entered " << iVar << endl;
                }
                break; 
            } 
            return 0; 
        } 
        break;

		case WM_KEYDOWN:
			key[wParam] = TRUE;
        break;

		case WM_KEYUP:
			key[wParam] = FALSE;
        break;

        case WM_CLOSE:
            Console.Close();
            DestroyWindow(hWnd);
        break;

        case WM_DESTROY:
            PostQuitMessage(0);
        break;

        default:
            return (DefWindowProc(hWnd, Msg, wParam, lParam));
    }

    return 0;
}

INT WINAPI WinMain( HINSTANCE  hInstance,
                    HINSTANCE  hPrevInstance,
                    LPSTR      lpCmdLine,
                    INT        nCmdShow )
{
    WNDCLASSEX    wc;

    wc.cbSize           = sizeof(WNDCLASSEX);
    wc.style            = 0;
    wc.lpfnWndProc      = (WNDPROC)WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = hInstance;
    wc.hIcon            = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
    wc.hIconSm          = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground    = (HBRUSH)(COLOR_WINDOW + 1);
    wc.lpszMenuName     = MAKEINTRESOURCE(IDR_MENU);
    wc.lpszClassName    = ClassName;

    if (!RegisterClassEx(&wc))
    {
        MessageBox(NULL, "Failed To Register The Window Class.", "Error", MB_OK | MB_ICONERROR);
        return 0;
    }

    HWND    hWnd;

    hWnd = CreateWindowEx(
    WS_EX_CLIENTEDGE,
    ClassName,
    "Console Debug Window",
    WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    260,
    120,
    NULL,
    NULL,
    hInstance,
    NULL);

    if (!hWnd)
    {
        MessageBox(NULL, "Window Creation Failed.", "Error", MB_OK | MB_ICONERROR);
        return 0;
    }

    ShowWindow(hWnd, SW_SHOW);
    UpdateWindow(hWnd);

    MSG    Msg;

	while (1)
	{
        while (PeekMessage(&Msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&Msg, NULL, 0, 0))
			{
				TranslateMessage(&Msg);
				DispatchMessage(&Msg);
			}
			else
                return Msg.wParam;
		}

        if (key['O'] && released_key['O'] == 0)
        {
            Console.Open();
            released_key['O'] = 1;
        }

        if (!key['O'])
            released_key['O'] = 0;

        if (key['C'] && released_key['C'] == 0)
        {
            Console.Close();
            released_key['C'] = 1;
        }

        if (!key['C'])
            released_key['C'] = 0;

        if (key['T'] && released_key['T'] == 0)
        {
            int iVar;

            // test stdio
            fprintf(stdout, "Test output to stdout\n");
            fprintf(stderr, "Test output to stderr\n");
            fprintf(stdout, "Enter an integer to test stdin: ");
            scanf("%d", &iVar);
            printf("You entered %d\n", iVar);

            //test iostreams
            cout << "Test output to cout" << endl;
            cerr << "Test output to cerr" << endl;
            clog << "Test output to clog" << endl;
            cout << "Enter an integer to test cin: ";
            cin >> iVar;
            cout << "You entered " << iVar << endl;

            released_key['T'] = 1;
        }

        if (!key['T'])
            released_key['T'] = 0;
    }
}
